﻿'# Title    : SlimDX ClipPlaneReflection
'# Author   : Sascha Bajus
'# Created  : 01/2009
'# Copyright © 2009 Sascha Bajus




#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region



Imports SlimDX
Imports SlimDX.Direct3D9
Imports SlimDX.DirectInput
Imports sdxBaseEngine



Public Class Project : Inherits SDX9BaseEngine


    Private _reflectionPlane As sdxClipPlane
    Private _reflection As sdxBillboard
    Private _teapot As Mesh
    Private _floor As sdxBillboard
    Private _mat(5) As Material
    Private _matindex As Integer = 0
    Private _light As New Light
    Private _camset As engineCamera.cameraSetMoving

    Private _matText() As String = {"Steel", "Copper", "Glass", "Gold", "Red Marble", "Mirror"}


    Public Sub New()

        AddHandler dxRenderScene, AddressOf render
        AddHandler dxKeyboardState, AddressOf keyboard
        AddHandler dxMouseState, AddressOf mouse
        AddHandler dxLoadObjectsSettings, AddressOf Loading
        AddHandler dxLoadObjectsTextures, AddressOf loadTextures
        AddHandler dxEngineClosed, AddressOf ExitAplication
        '---------------------------------------------

        '#start Engine
        If dxEngine("SlimDX - BaseEngine - ClipPlane Reflections") = False Then CloseBaseEngine()

    End Sub

#Region "Loading"

    Private Sub loadTextures()

        _reflection.addColormap(dxTexture.FromEngineResource("steel.png", Color.Magenta))
        _reflection.addColormap(dxTexture.FromEngineResource("copper.png", Color.Magenta))
        _reflection.addColormap(dxTexture.FromEngineResource("glass.png", Color.Magenta))
        _reflection.addColormap(dxTexture.FromEngineResource("gold.png", Color.Magenta))
        _reflection.addColormap(dxTexture.FromEngineResource("redmarble.png", Color.Magenta))
        _reflection.addColormap(dxTexture.FromEngineResource("mirror.png", Color.Magenta))

        _floor.addColormap(dxTexture.FromEngineResource("floor.png", Color.Magenta))
    End Sub

    Private Sub Loading()
        '#create Teapot
        _teapot = Mesh.CreateTeapot(dxDevice)

        '#create reflection Billboard and load Textures from resx
        dxEngine.ResourceLoader.LoadResourceFile = "data\resx_textures.dll"

        _reflection = New sdxBillboard(New Vector3(3.1, 3.1, 0), New Vector3(1, 0, 0))

        '#create Materials Diffuse for reflection Billboard
        _mat(0).Diffuse = Color.FromArgb(255, 10, 15, 10)    '#steel
        _mat(1).Diffuse = Color.FromArgb(255, 125, 125, 125) '#copper
        _mat(2).Diffuse = Color.FromArgb(255, 120, 120, 140) '#glass
        _mat(3).Diffuse = Color.FromArgb(255, 35, 25, 25)    '#gold
        _mat(4).Diffuse = Color.FromArgb(255, 105, 85, 85)   '#marble
        _mat(5).Diffuse = Color.FromArgb(255, 255, 255, 255) '#mirror

        '#create reflection Plane for Clipping
        _reflectionPlane = New sdxClipPlane(New Vector3(3, 3, 0), New Vector3(0, 0.05, 0))

        '#create Floor
        _floor = New sdxBillboard(New Vector3(10, 0, 10))


        '#LightSetup for Scene - Light0
        With _light
            .Type = LightType.Point
            .Ambient = Color.FromArgb(255, 20, 20, 20)
            .Diffuse = Color.FromArgb(255, 255, 255, 255)
            .Position = New Vector3(-11, 20, -3)
            .Range = 30
            .Attenuation1 = 0.05
        End With
        dxLightsFFP.LightAddNew(0) = _light

        '#LightSetup for Reflected Scene - Light1
        With _light
            .Position = Vector3.Reflect(New Vector3(-11, 20, -3), _reflection.NormalVector)
        End With
        dxLightsFFP.LightAddNew(1) = _light

        '#enable Lightning
        dxLightsFFP.EnableLighting = True
        '#enable ZBuffer
        dxZBuffer.Enabled = True

        '#create CamSet for automatic movement
        With _camset
            .Curved = False
            .StartPoint = New Vector3(0, 4, -5)
            .Target = New Vector3(0, 1.5, 0)
            .Movement = New Vector3(6, 0.9, -5)
            .Speed = New Vector2(80, 80)
        End With

        '#use Camset
        dxCamera.useCameraSet(_camset)
    End Sub

#End Region

#Region "## render ##"
    Private Sub render()

        '// Render reflected Scene to plane
        '// -------------------------------
        '// - Transform the plane to position 
        '// - Setup the ClipPlanes and enable Clipping. Turn CullMode to Cull.Clockwise  
        '// - Set the reflection material
        '// - Draw Objects with reflectionMatrix and reflected light
        '// - Disable Clipping

        '#setup for Clipping
        _reflectionPlane.TransformCoordinate = Matrix.Translation(New Vector3(0, 1.5, 3))
        _reflectionPlane.ClippingEnabled = True

        '#set reflection Material and light1
        dxDevice.Material = _mat(_matindex)
        dxLightsFFP.LightIndexEnable = 1

        '#Draw reflected Teapot
        dxTransformWorld = Matrix.Translation(New Vector3(0, 1.5, 0)) * dxMatrix.TimebasedRotationY(0.001) * _reflectionPlane.ReflectionMatrix
        _teapot.DrawSubset(0)

        '#Draw reflected Floor
        dxTransformWorld = _reflectionPlane.ReflectionMatrix
        _floor.setTexture(0, 0)
        _floor.draw(False)

        '#Disable Clipping
        _reflectionPlane.ClippingEnabled = False



        '// Render reflection Object
        '// ------------------------
        '// - Set the current Material Texture first
        '// - Draw the Reflection Object with default Emissive Material 
        '//   and activated Alphablending at the refelctionPlanes 
        '//   Position.
        '// - Disable Alphablending and Light1

        dxTexture.AlphaBlending2 = True
        dxTransformWorld = Matrix.Translation(New Vector3(0, 0, 3))
        _reflection.setTexture(0, _matindex)
        _reflection.draw()
        dxTexture.AlphaBlending2 = False
        dxLightsFFP.LightIndexDisable = 1


        '// Render Scene
        '// ------------------------
        '// - Render the Floor with default Emissive Material
        '// - Disable Texture Sampler
        '// - Render Teapot with default Material and light0

        '#Draw Floor
        dxTransformWorld = Matrix.Identity
        _floor.setTexture(0, 0)
        _floor.draw()

        '#Draw Teapot
        dxDevice.SetTexture(0, Nothing)
        dxLightsFFP.SetDefaultMaterial()
        dxLightsFFP.LightIndexEnable = 0
        dxTransformWorld = Matrix.Translation(New Vector3(0, 1.5, 0)) * dxMatrix.TimebasedRotationY(0.001)
        _teapot.DrawSubset(0)
        dxLightsFFP.LightIndexDisable = 0
        '#render info
        dx2DText.drawFPS(vbCrLf & "  Keys:  1 - 6 to switch Material" & vbCrLf & _
                         "  Material:  " & _matText(_matindex))
    End Sub
#End Region
#Region "## Inputdevices ##"
    Private Sub keyboard()
        With dxEngine

            '#Close and Dispose
            If .KeyPressed(Key.Escape) Then CloseBaseEngine()

            '#change reflection
            If .KeyAsSwitch(Key.D1) Then _matindex = 0
            If .KeyAsSwitch(Key.D2) Then _matindex = 1
            If .KeyAsSwitch(Key.D3) Then _matindex = 2
            If .KeyAsSwitch(Key.D4) Then _matindex = 3
            If .KeyAsSwitch(Key.D5) Then _matindex = 4
            If .KeyAsSwitch(Key.D6) Then _matindex = 5

        End With
    End Sub

    Private Sub mouse()
    End Sub
#End Region
#Region "## Exit Application ##"
    Private Sub ExitAplication()
        If Not _teapot Is Nothing Then _teapot.Dispose()
        If dxEngine.EngineError Then
            Me.Hide()
            MsgBox(dxEngine.EngineErrorDescription, MsgBoxStyle.Critical)
        End If
        End
    End Sub
#End Region

    Private Sub InitializeComponent()
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(Project))
        Me.SuspendLayout()
        '
        'Project
        '
        Me.ClientSize = New System.Drawing.Size(284, 262)
        Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
        Me.Name = "Project"
        Me.ResumeLayout(False)

    End Sub
End Class

